/*
  SPDX-FileCopyrightText: 2013-2021 Laurent Montel <montel@kde.org>

  SPDX-License-Identifier: GPL-2.0-only
*/

#pragma once

#include <QStringList>

namespace KSieveUi
{
namespace Test
{
QStringList fullCapabilitiesList();
}
}

