/*
   SPDX-FileCopyrightText: 2020-2021 Laurent Montel <montel@kde.org>

   SPDX-License-Identifier: LGPL-2.0-or-later
*/

#include "sieveimappasswordprovider.h"

using namespace KSieveUi;
SieveImapPasswordProvider::SieveImapPasswordProvider(QObject *parent)
    : QObject(parent)
{
}
