/*
   SPDX-FileCopyrightText: 2013-2021 Laurent Montel <montel@kde.org>

   SPDX-License-Identifier: LGPL-2.0-or-later
*/

#pragma once

#include "sievetemplatewidget.h"

namespace KSieveUi
{
namespace SieveDefaultTemplate
{
Q_REQUIRED_RESULT QVector<PimCommon::defaultTemplate> defaultTemplates();
}
}

